import datetime
import time
import PICommon_pb2
import PIDataScenario_pb2
import remoteapi

"""
TestLiveLogging.py

Unit test for Remote API that tests commands configuring SIMDIS
live data recording. Test will initialize a Scenario with one
Platform, and add data to it periodically to prompt action
from the Live Logging Plug-in.
"""
client = remoteapi.DiscoveryClient().createClient()

if client is None:
	raise Exception("Failed to connect.")

pi = client.pluginApi()
piData = pi.data()
piCommon = pi.common()

pi.config().setResponsesEnabled(True)

logFilePath = "$(TMP)/OkToDeleteThisFile.asi"

# delete the scenario
f = piData.deleteScenario()
if not f.isOk(5000):
	raise Exception("Failed to delete scenario")

# create a new live scenario to test live-only commands
f = piData.initializeScenario("Unit test live mode", 0, 0, True)
if not f.isOk(1000):
	raise Exception("Failed to initialize live scenario")
currDateTime = datetime.datetime.utcnow()
startOfYear = datetime.datetime(currDateTime.year,1,1)
secsSinceYear = (currDateTime - startOfYear).total_seconds()
f = piData.setReferenceYear(currDateTime.year)
if not f.isOk(1000):
	raise Exception("Failed to set reference year")
piData.clockSetMode(PIDataScenario_pb2.CLOCKMODE_FREEWHEEL, secsSinceYear)
piData.clockSetTime(secsSinceYear)

# Create a platform to add live data points to
f = piData.createPlatform("Unit test live platform", "x", 1)
if not f.isOk(1000):
	raise Exception("Failed to create a new platform")
platId = f.value()

# Test that configuring auto-start for the future does not start recording -----------------------------------
print("Begin Test 1")
currDateTime = datetime.datetime.utcnow()
startOfYear = datetime.datetime(currDateTime.year,1,1)
secsSinceYear = (currDateTime - startOfYear).total_seconds()
f = piCommon.startRecording(logFilePath, True, False, 1, False, secsSinceYear + 5)
if not f.isOk(1000):
	raise Exception("Failed to start recording")
if f.value() != PICommon_pb2.RECORDING_PENDING:
	raise Exception("startRecording() returned unexpected recording state: " + str(f.value()))

# Adding data before the start point should not trigger recording to start
f = piData.addPlatformData(platId, secsSinceYear, 0, 0, 0)

# Test stop recording without waiting, to make sure recording has not started
f = piCommon.stopRecording()
if not f.isOk(1000):
	raise Exception("Failed to stop recording")
if f.value() != PICommon_pb2.RECORDING_NOT_ACTIVE:
	raise Exception("stopRecording() returned unexpected recording state: " + str(f.value()))

# Test that recording will start at the configured auto-start time -----------------------------------
print("Begin Test 2")
currDateTime = datetime.datetime.utcnow()
startOfYear = datetime.datetime(currDateTime.year,1,1)
secsSinceYear = (currDateTime - startOfYear).total_seconds()
f = piCommon.startRecording(logFilePath, True, False, 1, False, secsSinceYear + 5)
if not f.isOk(1000):
	raise Exception("Failed to start recording")
if f.value() != PICommon_pb2.RECORDING_PENDING:
	raise Exception("startRecording() returned unexpected recording state: " + str(f.value()))

# Adding data after the auto-start time should trigger recording to start
time.sleep(6) # 6 seconds
f = piData.addPlatformData(platId, secsSinceYear, 0, 0, 0)

# Test stop recording after waiting, to make sure recording started
f = piCommon.stopRecording()
if not f.isOk(1000):
	raise Exception("Failed to stop recording")
if f.value() != PICommon_pb2.RECORDING_OK:
	raise Exception("stopRecording() returned unexpected recording state: " + str(f.value()))

# Test that recording starts when given a time in the past -----------------------------------
print("Begin Test 3")
# Do not reset currDateTime, should be in the past
f = piCommon.startRecording(logFilePath, True, False, 1, False, secsSinceYear)
if not f.isOk(1000):
	raise Exception("Failed to start recording")
if f.value() != PICommon_pb2.RECORDING_OK:
	raise Exception("startRecording() returned unexpected recording state: " + str(f.value()))

# Test stop recording
f = piCommon.stopRecording()
if not f.isOk(1000):
	raise Exception("Failed to stop recording")
if f.value() != PICommon_pb2.RECORDING_OK:
	raise Exception("stopRecording() returned unexpected recording state: " + str(f.value()))

# Test that recording halts after the configured halt time -----------------------------------
print("Begin Test 4")
currDateTime = datetime.datetime.utcnow()
startOfYear = datetime.datetime(currDateTime.year,1,1)
secsSinceYear = (currDateTime - startOfYear).total_seconds()
f = piCommon.startRecording(logFilePath, True, False, 1, False, secsSinceYear - 1, secsSinceYear + 5)
if not f.isOk(1000):
	raise Exception("Failed to start recording")
if f.value() != PICommon_pb2.RECORDING_OK:
	raise Exception("startRecording() returned unexpected recording state: " + str(f.value()))

# Adding data after the auto-halt time should trigger recording to halt
time.sleep(6) # 6 seconds
f = piData.addPlatformData(platId, secsSinceYear, 0, 0, 0)

f = piCommon.stopRecording()
if not f.isOk(1000):
	raise Exception("Failed to stop recording")
if f.value() != PICommon_pb2.RECORDING_NOT_ACTIVE:
	raise Exception("stopRecording() returned unexpected recording state: " + str(f.value()))

