import math
import RemoteApi_pb2
import remoteapi
import remoteapi.PluginApi
import ScenarioHelper
import time
import zlib
import struct

"""
TestPlatformTextureIcon.py

Unit test for Remote API Platform Texture Icon
"""

# Adapted from https://stackoverflow.com/questions/8554282
def makeGrayPNG(data, height = None, width = None):
	def I1(value):
		return struct.pack("!B", value & (2**8-1))
	def I4(value):
		return struct.pack("!I", value & (2**32-1))
	# compute width&height from data if not explicit
	if height is None:
		height = len(data) # rows
	if width is None:
		width = 0
		for row in data:
			if width < len(row):
				width = len(row)
	# generate these chunks depending on image type
	makeIHDR = True
	makeIDAT = True
	makeIEND = True
	png = b"\x89" + "PNG\r\n\x1A\n".encode('ascii')
	if makeIHDR:
		colortype = 0 # true gray image (no palette)
		bitdepth = 8 # with one byte per pixel (0..255)
		compression = 0 # zlib (no choice here)
		filtertype = 0 # adaptive (each scanline seperately)
		interlaced = 0 # no
		IHDR = I4(width) + I4(height) + I1(bitdepth)
		IHDR += I1(colortype) + I1(compression)
		IHDR += I1(filtertype) + I1(interlaced)
		block = "IHDR".encode('ascii') + IHDR
		png += I4(len(IHDR)) + block + I4(zlib.crc32(block))
	if makeIDAT:
		raw = b""
		for y in range(height):
			raw += b"\0" # no filter for this scanline
			for x in range(width):
				c = b"\0" # default black pixel
				if y < len(data) and x < len(data[y]):
					c = I1(data[y][x])
				raw += c
		compressor = zlib.compressobj()
		compressed = compressor.compress(raw)
		compressed += compressor.flush() #!!
		block = "IDAT".encode('ascii') + compressed
		png += I4(len(compressed)) + block + I4(zlib.crc32(block))
	if makeIEND:
		block = "IEND".encode('ascii')
		png += I4(0) + block + I4(zlib.crc32(block))
	return png

# Make 4x4 icons to avoid rescaling
crossPng = makeGrayPNG([[0,255,255,0],[255,255,255,255],[255,255,255,255],[0,255,255,0]])
xPng = makeGrayPNG([[255,0,0,255],[0,255,255,0],[0,255,255,0],[255,0,0,255]])

client = remoteapi.DiscoveryClient().createClient()
try:
	if client is None:
		raise Exception("Failed to connect.")

	pi = client.pluginApi()
	piData = pi.data()
	piSimdis = pi.simdis()

	ScenarioHelper.newScenario(piData, origin=[math.radians(22), math.radians(-159),0])
	f = ScenarioHelper.createPlatform(client, 20, [math.radians(22.1), math.radians(-159), 0.0]) # slightly north
	if not f.isOk(2500):
		raise Exception("Failed createPlatform: " + f.responseDescription())
	platId = f.value().piData.platform.uniqueId

	# Set a reasonable view
	piSimdis.tetherCamera("MainView", platId)
	piSimdis.setCameraRAE("MainView", 50.0, 0.0, 0.3)

	# Create a texture
	f = piSimdis.addTextureStream(crossPng, 0, "Python Image")
	if not f.isOk(2500):
		raise Exception("Failed addTextureStream: " + f.responseDescription())
	textureId = f.value()

	# Assign the texture to the platform
	f = piSimdis.setPlatformTextureIcon(platId, textureId)
	if not f.isOk(2500):
		raise Exception("Failed setPlatformTextureIcon: " + f.responseDescription())
	print("Check SIMDIS; you should see a + icon on the platform.  Waiting 3 seconds...")
	time.sleep(3.0)

	# Attempt to update the PNG icon
	f = piSimdis.updateTextureStream(textureId, xPng, 0, 0)
	if not f.isOk(2500):
		raise Exception("Failed updateTextureStream: " + f.responseDescription())
	print("Check SIMDIS; you should see an X icon on the platform.  Waiting 3 seconds...")
	time.sleep(3.0)

	# Update the texture with a dark grey band 3x2 along the top-right
	greySquare = bytes(bytearray.fromhex('404040404040'))
	GL_TEXTURE_2D = 0x0DE1
	GL_LUMINANCE = 0x1909
	GL_UNSIGNED_BYTE = 0x1401
	f = piSimdis.updateTexture(textureId, GL_TEXTURE_2D, 0, 1, 2, 3, 2, GL_LUMINANCE, GL_UNSIGNED_BYTE, greySquare)
	if not f.isOk(2500):
		raise Exception("Failed updateTexture: " + f.responseDescription())
	print("Check SIMDIS; the X should have a grey panel on top-right corner.")

	# Clean up the texture memory
	f = piSimdis.removeTexture(textureId)
	if not f.isOk(2500):
		raise Exception("Failed removeTexture: " + f.responseDescription())

finally:
	client.close()
