from math import radians
import PICommon_pb2
import remoteapi

"""

TestProjector.py

Unit test for Remote API that works with projectors; expects a file created by Simple Server
NOTE: Start Simple Server in File Mode before executing this test
"""

client = remoteapi.DiscoveryClient().createClient()

if client is None:
	raise Exception("Failed to connect.")

pi = client.pluginApi()
pi.config().setResponsesEnabled(True)
piData = pi.data()

# Verify that a scenario is loaded
f = piData.isScenarioInitialized()
if not f.isOk(1000):
	raise Exception("isScenarioInitialized failed: " + f.responseDescription())

if not f.value():
	raise Exception("A Simple Server scenario needs to be loaded")

# Find a beam to add a projector to
f = piData.findBeam("11 p-3c_orion_nrl beam");
if not f.isOk(1000):
	raise Exception("findBeam: " + f.responseDescription())

if f.value() == 0:
	raise Exception("A Simple Server scenario needs to be loaded")

# Add a projector
host = f.value()
projectorName = "ProjectorTest"
originalId = 47
f = piData.createProjector(host, projectorName, originalId)
if not f.isOk(1000):
	raise Exception("createProjector: " + f.responseDescription())
projector = f.value();

# Verify the projector was added
f = piData.findEntity(projectorName)
if not f.isOk(1000):
	raise Exception("findEntity: " + f.responseDescription())
if f.value() != projector:
	raise Exception("findEntity: ID did not match expected = " + str(projector) + " got = " + str(f.value()))
f = piData.getProjectorHeader(projector)
if not f.isOk(1000):
	raise Exception("getProjectorHeader: " + f.responseDescription())
if f.value().name != projectorName:
	raise Exception("getProjectorHeader: name did not match expected = " + projectorName + " got = " + f.value().name)
if f.value().originalId != originalId:
	raise Exception("getProjectorHeader: original ID did not match expected = " + originalId + " got = " + f.value().originalId)

# Get the start time
f = piData.clockGetBegin();
if not f.isOk(1000):
	raise Exception("clockGetBegin: " + f.responseDescription())
startTime = f.value();

# Add some data to the projector
f = piData.addProjectorFov(projector, startTime, 1.0)
if not f.isOk(1000):
	raise Exception("addProjectorFov: " + f.responseDescription())

f = piData.addProjectorFov(projector, startTime+1.0, 2.0)
if not f.isOk(1000):
	raise Exception("addProjectorFov: " + f.responseDescription())

# Verify the data
f = piData.getAllProjectorData(projector)
if not f.isOk(1000):
	raise Exception("getProjectorData: " + f.responseDescription())
if len(f.value()) != 2:
	raise Exception("getProjectorData: got the wrong length")
if f.value()[0].time != startTime:
	raise Exception("getProjectorData: got the wrong start time")
# addProjectorFov expects degrees while getProjectorData returns radians
if f.value()[0].fov != radians(1.0):
	raise Exception("getProjectorData: got the wrong start value")
if f.value()[1].time != startTime+1.0:
	raise Exception("getProjectorData: got the wrong end time")
if f.value()[1].fov != radians(2.0):
	raise Exception("getProjectorData: got the wrong end value")

# Get data by specifying the times
f = piData.getProjectorData(projector, startTime, startTime+1.0)
if not f.isOk(1000):
	raise Exception("getProjectorData: " + f.responseDescription())
if len(f.value()) != 2:
	raise Exception("getProjectorData: got the wrong length")

# Get the first point
f = piData.getProjectorData(projector, startTime, startTime+0.5)
if not f.isOk(1000):
	raise Exception("getProjectorData: " + f.responseDescription())
if len(f.value()) != 1:
	raise Exception("getProjectorData: got the wrong length")

# Get the second point
f = piData.getProjectorData(projector, startTime+0.5, startTime+1.0)
if not f.isOk(1000):
	raise Exception("getProjectorData: " + f.responseDescription())
if len(f.value()) != 1:
	raise Exception("getProjectorData: got the wrong length")

# Get no points by specifying an out of bounds time range
f = piData.getProjectorData(projector, startTime+1.5, startTime+2.0)
if not f.isOk(1000):
	raise Exception("getProjectorData: " + f.responseDescription())
if len(f.value()) != 0:
	raise Exception("getProjectorData: got the wrong length")

# Add some status
f = piData.setStatus(projector, startTime, True)
if not f.isOk(1000):
	raise Exception("setStatus: " + f.responseDescription())

f = piData.setStatus(projector, startTime+2.0, False)
if not f.isOk(1000):
	raise Exception("setStatus: " + f.responseDescription())

# Verify the status
f = piData.getStatus(projector, startTime, startTime+2.0, False)
if not f.isOk(1000):
	raise Exception("getStatus: " + f.responseDescription())
if len(f.value()) != 2:
	raise Exception("getStatus: got the wrong length")
if f.value()[0].time != startTime:
	raise Exception("getStatus: got the wrong start time")
if f.value()[0].status != True:
	raise Exception("getStatus: got the wrong start value")
if f.value()[1].time != startTime+2.0:
	raise Exception("getStatus: got the wrong end time")
if f.value()[1].status != False:
	raise Exception("getStatus: got the wrong end value")

# Get the first point
f = piData.getStatus(projector, startTime, startTime+1.0, False)
if not f.isOk(1000):
	raise Exception("getStatus: " + f.responseDescription())
if len(f.value()) != 1:
	raise Exception("getStatus: got the wrong length")

# Get the second point
f = piData.getStatus(projector, startTime+1.0, startTime+2.0, False)
if not f.isOk(1000):
	raise Exception("getStatus: " + f.responseDescription())
if len(f.value()) != 1:
	raise Exception("getStatus: got the wrong length")

# Get the second point, but with initial value
f = piData.getStatus(projector, startTime+1.0, startTime+2.0, True)
if not f.isOk(1000):
	raise Exception("getStatus: " + f.responseDescription())
if len(f.value()) != 2:
	raise Exception("getStatus: got the wrong length")
if f.value()[0].time != startTime+1.0:
	raise Exception("getStatus: got the wrong start time")
if f.value()[0].status != True:
	raise Exception("getStatus: got the wrong start value")

# Add raster file
inFilename = "compass.png"
f = piData.setProjectorRasterFilename(projector, inFilename)
if not f.isOk(1000):
	raise Exception("setProjectorRasterFilename: " + f.responseDescription())

# Verify raster file
f = piData.getProjectorRasterFilename(projector)
if not f.isOk(1000):
	raise Exception("getProjectorRasterFilename: " + f.responseDescription())
outFilename = f.value();
if inFilename != outFilename:
	raise Exception("Names do not match: in = " + inFilename + " out = " + outFilename)

# Cleanup
f = piData.removeEntity(projector)
if not f.isOk(1000):
	raise Exception("removeEntity: " + f.responseDescription())

client.close()
print("Test successful")
